#include "stdafx.h"
#include "Cubes.h"
#include "CubesDoc.h"
#include "CubesView.h"
#include "VisualObject.h"
#include "ModelSetManager.h"   // sets of models treated as one
#include "InfoViewManager.h"
#include "log.h"

// for treating a set of models as one
static ModelSetManager msm;

void InfoViewManager::ForceModelSwitch() {
	msm.forceNext();
}

void InfoViewManager::ModelSwitch() {
	//log_message("Entered ModelSwitch" );
	if ( msm.next() ) {
		currentIdx = lookup( msm.getName() );
		msm.registerIdx( currentIdx );
		//log_messagei1( "currentIdx now:", currentIdx );
	} else {
		//log_message("..did not switch anything..");
	}
}

/**
 * ======== InfoViewManager::InfoViewManager ========
 *
 *  Purpose:
 *    Class constructor, sets default change units to 10%
 */

InfoViewManager::InfoViewManager() {
	transformIdx = 0;
	currentIdx = -1;
	internalMax = 0;
	markerIdx = -1;
	resolution = 65535;

	// change units
	fade_units = 10;
	scale_units = 10;
	location_units = 10;

	xMin = 0;
	xMax = 0;
	yMin = 0;
	yMax = 0;
	zMin = 0;
	zMax = 0;

	currentProperty = NULL;
	clearCurrentGroup();
}

void InfoViewManager::SpaceX( char* value ) {
	float f = (float)atof( value );
	int numberObjects = msm.getNumberObjects();
	int saveIdx = currentIdx;
	float originalX = 0;
	for ( int i = 0; i < numberObjects; i++ ) {
		currentIdx = lookup( msm.getObjectAt( i ));
		if ( i == 0 ) {
			originalX = getXlocation();
		}
		float newx = originalX + i * f;
		setXlocation( newx );
	}
	currentIdx = saveIdx;
}

void InfoViewManager::SpaceY( char* value ) {
	float f = (float)atof( value );
	int numberObjects = msm.getNumberObjects();
	int saveIdx = currentIdx;
	float originalY = 0;
	for ( int i = 0; i < numberObjects; i++ ) {
		currentIdx = lookup( msm.getObjectAt( i ));
		if ( i == 0 ) {
			originalY = getYlocation();
		}
		float newy = originalY + i * f;
		setYlocation( newy );
	}
	currentIdx = saveIdx;
}
void InfoViewManager::SpaceZ( char* value ) {
	float f = (float)atof( value );
	int numberObjects = msm.getNumberObjects();
	int saveIdx = currentIdx;
	float originalZ = 0;
	for ( int i = 0; i < numberObjects; i++ ) {
		currentIdx = lookup( msm.getObjectAt( i ));
		if ( i == 0 ) {
			originalZ = getZlocation();
		}
		float newz = originalZ + i * f;
		setZlocation( newz );
	}
	currentIdx = saveIdx;
}

void InfoViewManager::resetVisuals() {
	for ( int i = 0; i < transformIdx; i++ ) {
		VisualObject* vo = visualObjects[i];
		vo->resetOriginalValues();
	}
}

void InfoViewManager::RegisterModelSet( char* name, int count ) {
	msm.RegisterModelSet( new ModelSet( name, count ));
}

void InfoViewManager::saveState( FILE* fp ) {
	// save each object location, scaling, rotation, transparency, texture
	// so a configuration can be reloaded.
	// probably should save the file(s) this was all loaded from as well so state can be contained
	// in a single file.
	for ( int i = 0; i < transformIdx; i++ ) {
		fprintf( fp, "%s\n", transformNameList[i] );
		VisualObject* vo = visualObjects[i];
		ISFVec3fObject* m_pTranslation = vo->getTranslation();
		float x,y,z;
		if ( SUCCEEDED ( m_pTranslation->get_X( &x )) && SUCCEEDED( m_pTranslation->get_Y( &y )) && SUCCEEDED( m_pTranslation->get_Z( &z ))) {
			fprintf( fp, "%f %f %f\n", x, y, z );
		} else {
			fprintf( fp, "0 0 0\n" );
		}
		ISFVec3fObject* m_pScale = vo->getScale();
		if ( SUCCEEDED( m_pScale->get_X( &x )) && SUCCEEDED( m_pScale->get_Y( &y )) && SUCCEEDED( m_pScale->get_Z( &z ))) {
			fprintf( fp, "%f %f %f\n", x, y, z );
		} else {
			fprintf( fp, "0 0 0\n" );
		}
		ISFFloatObject* m_pTransparency = vo->getTransparency();
		float transparency = 0.0f;
		if (( m_pTransparency != NULL ) && SUCCEEDED( m_pTransparency->get_Value( &transparency ))) {
			fprintf( fp, "%f\n", transparency );
		} else {
			fprintf( fp, "0\n" );
		}
		ISFColorObject* color = vo->getDiffuseColor();
		float r, g, b;
		if (( color != NULL ) && SUCCEEDED( color->get_Red( &r )) && SUCCEEDED( color->get_Green( &g )) && SUCCEEDED( color->get_Blue( &b ))) {
			fprintf( fp, "%f %f %f\n", r, g, b );
		} else {
			fprintf( fp, "0 0 0\n" );
		}
		color = vo->getEmissiveColor();
		if (( color != NULL ) && SUCCEEDED( color->get_Red( &r )) && SUCCEEDED( color->get_Green( &g )) && SUCCEEDED( color->get_Blue( &b ))) {
			fprintf( fp, "%f %f %f\n", r, g, b );
		} else {
			fprintf( fp, "0 0 0\n" );
		}

		ISFRotationObject* rotation = vo->getRotation();
		float angle;
		if (( rotation != NULL ) && SUCCEEDED( rotation->get_X( &x )) && SUCCEEDED( rotation->get_Y( &y )) && SUCCEEDED( rotation->get_Z( &z )) && SUCCEEDED( rotation->get_Angle( &angle ))) {
			fprintf( fp, "%f %f %f\n", x, y, z );
			fprintf( fp, "%f\n", angle );
		} else {
			fprintf( fp, "0 0 0\n" );
			fprintf( fp, "0\n" );
		}

		if (vo->getLabelString() != NULL ) {
			fprintf( fp, "%s\n", vo->getLabelString() );
		} else {
			fprintf( fp, "\n" );
		}
	}
}

bool InfoViewManager::get3( char* line, float* x, float* y, float* z ) {
	char* scanner = line;
	char* v1 = scanner;
	while ( *scanner && (*scanner != ' ')) {
		scanner++;
	}
	if ( *scanner == ' ' ) {
		*scanner = 0;
		scanner++;
	} else {
		return( false );
	}
	char* v2 = scanner;
	while ( *scanner && (*scanner != ' ')) {
		scanner++;
	}
	if ( *scanner == ' ') {
		*scanner = 0;
		scanner++;
	} else {
		return( false );
	}
	char* v3 = scanner;
	*x = (float)atof( v1 );
	*y = (float)atof( v2 );
	*z = (float)atof( v3 );
	return( true );
}



void InfoViewManager::loadState( FILE* fp, int version ) {
	if (( version != 1 ) && ( version != 2 )) {
		log_messagei1( "Unexpected version", version );
	} else {
		log_message( "Loading state information" );
		char* line = NULL;
		while (( line = file_line( fp )) != NULL ) {
			int idx = lookup( line );
			// if the named object is found, set its location and scale
			if ( idx >= 0 ) { 
				free( line );
				float x, y, z;
				line = file_line( fp );
				if ( line != NULL ) {
					if ( get3( line, &x, &y, &z )) {
						setLocation( idx, x, y, z );
					}
					free( line );
				}
				line = file_line( fp );
				if ( line != NULL ) {
					if ( get3( line, &x, &y, &z )) {
						setScale( idx, x, y, z );
					}
					free( line );
				}
				line = file_line( fp );
				float transparency;
				if ( line != NULL ) {
					transparency = (float)atof( line );
					setTransparency( idx, transparency );
					free( line );
				}
				line = file_line( fp );
				float r, g, b;
				if ( line != NULL ) {
					if ( get3( line, &r, &g, &b )) {
						setDiffuseColor( idx, r, g, b );
					}
					free( line );
				}
				line = file_line( fp );
				if ( line != NULL ) {
					if ( get3( line, &r, &g, &b )) {
						setEmissiveColor( idx, r, g, b );
					}
					free( line );
				}
				line = file_line( fp );
				if ( line != NULL ) {
					if ( get3( line, &x, &y, &z )) {
						char* line2 = file_line( fp );
						if ( line2 != NULL ) {
							float angle = (float)atof( line2 );
							free( line2 );
							setRotation( idx, x, y, z, angle );
						}
					}
					free( line );
				}

				if ( version >= 2 ) {
					line = file_line( fp );
					if ( line != NULL ) {
						setLabel( idx, line );
					}
				}
			}
		}
	}
}


void InfoViewManager::clearCurrentGroup() {
	currentGroup = "__root__";
}

void InfoViewManager::Clear() {
	transformIdx = 0;
	currentIdx = -1;
	clearCurrentGroup();
}

void InfoViewManager::SetGroup() {
	if ( currentIdx != -1 ) {
		currentGroup = transformNameList[ currentIdx ];
		log_message1( "SetGroup SET CURRENT GROUP TO ", currentGroup );
	} else {
		log_message( "SetGroup DID NOT SET CURRENT GROUP" );
	}
}

void InfoViewManager::ClearGroup() {
	currentGroup = "__root__";
}

IMFNodeObject* InfoViewManager::getGroup() {
	
	if ( strcmp( currentGroup, "__root__" ) != 0 ) {
		log_message1( "currentGroup is ", currentGroup );
		int groupIdx = getIdx( currentGroup );
		if ( groupIdx != -1 ) {
			log_message( "OK, I found it" );
			return( visualObjects[ groupIdx ] -> getGroup() );
		} else {
			log_message( "OK, I did not find it" );
		}
	}
	return( NULL );
}


/**
 * ======== InfoViewManager::Cycle ========
 *
 *  Purpose:  
 *    Cycle to the next object.
 */

void InfoViewManager::Cycle() {
	if ( transformIdx > 0 ) {
		currentIdx++;
		if ( currentIdx >= transformIdx ) {
			currentIdx = 0;
		}
		log_message1( "object cycled to: ", transformNameList[ currentIdx ] );
	}
}


/**
 * ======== InfoViewManager::FadeIn ========
 *
 *  Purpose:
 *    Decrease the transparency (i.e. make more "solid") of the current object.
 */

void InfoViewManager::FadeIn() {
	log_message1( "FadeIn called", transformNameList[ currentIdx ] );
	if ( currentIdx >= 0 ) {
		ISFFloatObject* f = visualObjects[ currentIdx ]->getTransparency();
		if ( f != NULL ) {
			float value = 0.0f;
			f->get_Value(&value);
			value -= 0.01f * fade_units;
			if ( value < 0 ) {
				value = 0.0f;
			}
			f->put_Value(value);
		}
	}
}

void InfoViewManager::setTransparency( int idx, float value ) {
	ISFFloatObject* f = visualObjects[ idx ]->getTransparency();
	if ( f != NULL ) {
		//log_messagei1( "Setting transparency, idx:", idx );
		f->put_Value(value);
	} else {
		//log_messagei1( "Not setting transparency, idx: " , idx );
	}
}


/**
 * ======== InfoViewManager::FadeOut ========
 *
 *  Purpose:
 *    Increase the transparency (i.e. make it more "see through") of the current object.
 */

void InfoViewManager::FadeOut() {
	//log_message1( "FadeOut called", transformNameList[ currentIdx ] );
	if ( currentIdx >= 0 ) {
		//log_message( "GOT HERE" );
		ISFFloatObject* f = visualObjects[ currentIdx ]->getTransparency();
		if ( f != NULL ) {
			float value = 1.0f;
			f->get_Value(&value);
			value += 0.01f * fade_units;
			if ( value > 1.0f ) {
				value = 1.0f;
			}
			f->put_Value(value);
		}
	}
}

void InfoViewManager::SetFadeUnits( char* strvalue ) {
	if ( strvalue != NULL ) {
		fade_units = atoi( strvalue );
	}
}
void InfoViewManager::SetScaleUnits( char* strvalue ) {
	if ( strvalue != NULL ) {
		scale_units = atoi( strvalue );
	}
}

void InfoViewManager::SetMax( char* maxstr ) {
	internalMax = atoi( maxstr );
}


/**
 *  ======== InfoViewManager::SetObject ========
 *
 *  Purpose:
 *    Generic method for assigning all subsequent commands to a specific object.
 *    When accessed from outside, the three methods "SetObject", "SetProperty",
 *    and "SetValue" are called to change object attributes.
 *
 *  Input Parmaeters:
 *    name -- name of the object to associate all future commands with.
 *
 *  Return Value:
 *    none.
 */

void InfoViewManager::SetObject( char* name ) {
	char* msmName = msm.xlateName( name );
	if ( msmName != NULL ) {
		currentIdx = lookup( msmName );
		//log_messagei1( "calling msm register idx", currentIdx );
		msm.registerIdx( currentIdx );
	} else {
		currentIdx = lookup( name );
	}

	//log_message1( "SetObject: ", name );
	//log_messagei1( "..value: ", currentIdx );
}


/**
 *  ======== InfoViewManager::SetProperty ========
 *
 *  Purpose:
 *    Generic method for setting the type of property a subsequent "SetValue"
 *    method applies to.
 *
 *  Input Parameters:
 *    propertyName -- name of the property we are to associate with all future
 *      "SetValue" method calls.
 *
 *  Return Value:
 *    none.
 */

void InfoViewManager::SetProperty( char* propertyName ) {
	//log_message1( "SetProperty: ", propertyName );
	if ( currentProperty != NULL ) {
		free( currentProperty );
	}
	currentProperty = newstrcpy( propertyName );
}


/**
 *  ======== InfoViewManager::SetValue ========
 *
 *  Purpose:
 *    Generic method for setting absolute values.  Which value gets
 *    set depends on last call to "SetProperty", which sets the
 *    "currentProperty" variable.
 *
 *  Input Parameters:
 *    valueStr -- the value string passed in.  Since this was designed
 *      for interoperability with "neattools", in most cases this will
 *      be a string form of a value from 0 to 65535.
 *
 *  Return Value:
 *    none
 */

void InfoViewManager::SetValue( char* valueStr ) {
	//log_message1( "SetValue: ", valueStr );
	if ( currentProperty == NULL ) {
		return;
	}
	if ( currentIdx < 0 ) {
		return;
	}
	//log_messagei1( "calling msm register value", currentIdx );
	msm.registerValue( currentIdx );
	int ival = atoi( valueStr );
	if ( strcmp( currentProperty, "transparency" ) == 0 ) {
		int integerPercent = ival/655;
		if (( integerPercent < 0 ) || ( integerPercent > 100 )) {
			integerPercent = 100;
		} 
		ISFFloatObject* f = visualObjects[ currentIdx ]->getTransparency();
		if ( f != NULL ) {
			float value = 0.01f * integerPercent;
			if ( value > 1.0f ) {
				value = 1.0f;
			}
			f->put_Value(value);
		}
	} else if ( strcmp( currentProperty, "x" ) == 0 ) {
		float addition = (float)ival/(float)resolution * ( xMax - xMin );
		setXlocation( xMin + addition );
	} else if ( strcmp( currentProperty, "y" ) == 0 ) {
		float addition = (float)ival/(float)resolution * ( yMax - yMin );
		setYlocation( yMin + addition );
	} else if ( strcmp( currentProperty, "z" ) == 0 ) {
		float addition = (float)ival/(float)resolution * ( zMax - zMin );
		setZlocation( zMin + addition );
	} else if ( strcmp( currentProperty, "red" ) == 0 ) {
		setRed( ival );
	} else if ( strcmp( currentProperty, "green" ) == 0 ) {
		setGreen( ival );
	} else if ( strcmp( currentProperty, "blue" ) == 0 ) {
		setBlue( ival );
	} else if ( strcmp( currentProperty, "xscale" ) == 0 ) {
		setXscale( ival );
	} else if ( strcmp( currentProperty, "yscale" ) == 0 ) {
		setYscale( ival );
	} else if ( strcmp( currentProperty, "zscale" ) == 0 ) {
		setZscale( ival );
	} else if ( strcmp( currentProperty, "scale" ) == 0 ) {
		setScale( ival );
	} else if ( strcmp( currentProperty, "rotation" ) == 0 ) {
		setRotation( ival );
	} else if ( strcmp( currentProperty, "ins1" ) == 0 ) {
		setIns1( valueStr );
	} else if ( strcmp( currentProperty, "in1" ) == 0 ) {
		setIn1( ival );
	} else if ( strcmp( currentProperty, "in2" ) == 0 ) {
		setIn2( ival );
	} else if ( strcmp( currentProperty, "in3" ) == 0 ) {
		setIn3( ival );
	} else if ( strcmp( currentProperty, "in4" ) == 0 ) {
		setIn4( ival );
	}
}

float InfoViewManager::getXlocation() {
	VisualObject* vo = visualObjects[currentIdx];
	ISFVec3fObject* m_pTranslation = vo->getTranslation();
	float result;
	if ( SUCCEEDED( m_pTranslation->get_X( &result ))) {
		return( result );
	} else {
		return( 0 );
	}
}
float InfoViewManager::getYlocation() {
	VisualObject* vo = visualObjects[currentIdx];
	ISFVec3fObject* m_pTranslation = vo->getTranslation();
	float result;
	if ( SUCCEEDED( m_pTranslation->get_Y( &result ))) {
		return( result );
	} else {
		return( 0 );
	}
}
float InfoViewManager::getZlocation() {
	VisualObject* vo = visualObjects[currentIdx];
	ISFVec3fObject* m_pTranslation = vo->getTranslation();
	float result;
	if ( SUCCEEDED( m_pTranslation->get_Z( &result ))) {
		return( result );
	} else {
		return( 0 );
	}
}

void InfoViewManager::setLocation( int idx, float x, float y, float z ) {
	VisualObject* vo = visualObjects[currentIdx];
	ISFVec3fObject* m_pTranslation = vo->getTranslation();
	m_pTranslation->put_X( x );
	m_pTranslation->put_Y( y );
	m_pTranslation->put_Z( z );
}


void InfoViewManager::setScale( int idx, float x, float y, float z ) {
	VisualObject* vo = visualObjects[idx];
	//log_messagei1( "setScale", idx );
	if ( vo != NULL ) {
		//log_message( "got visual object" );
		ISFVec3fObject* m_pScale = vo->getScale();
		if ( m_pScale != NULL ) {
			//log_message( "putting values" );
			m_pScale->put_X( x );
			m_pScale->put_Y( y );
			m_pScale->put_Z( z );
		}
	}
}


/**
 *  Link two objects as siblings.  Somehow we need to also link the type of operation
 *  that travels down these sibling links.
 */
void InfoViewManager::linkObjects( char* object1, char* object2 ) {
	log_message1("First object in link: ", object1 );
	log_message1("Second object in link: ", object2 );
	int object1idx = lookup( object1 );
	int object2idx = lookup( object2 );
	if (( object1idx != -1 ) && ( object2idx != -1 )) {
		VisualObject* o1 = visualObjects[ object1idx ];
		VisualObject* o2 = visualObjects[ object2idx ];
		o1->setNextLink( o2 );
		log_message( "Objects have been linked" );
	} else {
		if ( object1idx == -1 ) log_message( "did not find first object" );
		if ( object2idx == -1 ) log_message( "did not find second object" );
	}
}

void InfoViewManager::setLabel( int idx, char* line ) {
	VisualObject* vo = visualObjects[ idx];
	if ( vo != NULL ) {
		vo->setLabel( line );
	}
}



void InfoViewManager::MarkX( char* objectName ) {
	if ( currentIdx >= 0 ) {
		MarkX( lookup( objectName ));
	}
}

void InfoViewManager::MarkX( int idx ) {
	if (( idx >= 0 ) && ( currentIdx != idx )) {
		VisualObject* vo = visualObjects[ idx ];
		ISFVec3fObject* markLocation = vo->getMarkerLocation();
		if ( markLocation != NULL ) {
			float x;
			if ( SUCCEEDED( markLocation->get_X( &x ))) {
				setXlocation( x );
			}
		} else log_message( "did not find markLocation" );
	}
}

void InfoViewManager::MarkY( char* objectName ) {
	if ( currentIdx >= 0 ) {
		MarkY( lookup( objectName ));
	}
}

void InfoViewManager::MarkY( int idx ) {
	if (( idx >= 0 ) && ( currentIdx != idx )) {
		VisualObject* vo = visualObjects[ idx ];
		ISFVec3fObject* markLocation = vo->getMarkerLocation();
		if ( markLocation != NULL ) {
			float y;
			if ( SUCCEEDED( markLocation->get_Y( &y ) ) ) {
				setYlocation( y );
			}
		}
	}
}
void InfoViewManager::MarkZ( char* objectName ) {
	if ( currentIdx >= 0 ) {
		MarkZ( lookup( objectName ));
	}
}

void InfoViewManager::MarkZ( int idx ) {
	if (( idx >= 0 ) && ( currentIdx != idx )) {
		VisualObject* vo = visualObjects[ idx ];
		ISFVec3fObject* markLocation = vo->getMarkerLocation();
		if ( markLocation != NULL ) {
			float z;
			if ( SUCCEEDED( markLocation->get_Z( &z ))) {
				setZlocation( z );
			}
		}
	}
}

void InfoViewManager::setXlocation( float value ) {
	if ( currentIdx >= 0 ) {	
		VisualObject* vo = visualObjects[currentIdx];	
		while ( vo != NULL ) {
			ISFVec3fObject* m_pTranslation = vo->getTranslation();
			//log_messagei1( "currentIdx", currentIdx );
			if (m_pTranslation == NULL ) {
				log_message("HEY, no X location!" );
			} else {
				m_pTranslation->put_X( value );
			}
			vo = vo->getNext();
		}
		//log_messagef( "putting X location", value );
	}
}
void InfoViewManager::setYlocation( float value ) {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];	
		while ( vo != NULL ) {
			ISFVec3fObject* m_pTranslation = vo->getTranslation();
			if ( m_pTranslation != NULL ) {
				m_pTranslation->put_Y( value );
			}
			vo = vo->getNext();
		}
		//log_messagef( "putting Y location", value );
	}
}
void InfoViewManager::setZlocation( float value ) {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];	
		while ( vo != NULL ) {
			ISFVec3fObject* m_pTranslation = vo->getTranslation();
			if ( m_pTranslation != NULL ) {
				m_pTranslation->put_Z( value );
			}
			vo = vo->getNext();
		}
		//log_messagef( "putting Z location", value );
	}
}

void InfoViewManager::setDiffuseColor( int idx, float r, float g, float b ) {
	ISFColorObject* color = visualObjects[ idx ]->getDiffuseColor();
	if ( color != NULL ) {
		color->put_Red( r );
		color->put_Green( g );
		color->put_Blue( b );
	}
}
void InfoViewManager::setEmissiveColor( int idx, float r, float g, float b ) {
	ISFColorObject* color = visualObjects[ idx ]->getEmissiveColor();
	if ( color != NULL ) {
		color->put_Red( r );
		color->put_Green( g );
		color->put_Blue( b );
	}
}

void InfoViewManager::setRed( int ival ) {
    ISFColorObject* m_pDiffuseColor = getDiffuseColor();
	if ( m_pDiffuseColor == NULL ) {
		m_pDiffuseColor = getEmissiveColor();
		if ( m_pDiffuseColor == NULL ) {
			return;
		}
	}

    float r;

    if ( SUCCEEDED( m_pDiffuseColor->get_Red( &r ))) {
		r = (float)ival/(float)65535;
		m_pDiffuseColor -> put_Red ( r );
	}
}

void InfoViewManager::setGreen( int ival ) {
    ISFColorObject* m_pDiffuseColor = getDiffuseColor();
	if ( m_pDiffuseColor == NULL ) {
		m_pDiffuseColor = getEmissiveColor();
		if ( m_pDiffuseColor == NULL ) {
			return;
		}
	}

    float g;

    if ( SUCCEEDED( m_pDiffuseColor->get_Green( &g ))) {
		g = (float)ival/(float)65535;
		m_pDiffuseColor -> put_Green ( g );
	}
}

void InfoViewManager::setBlue( int ival ) {
    ISFColorObject* m_pDiffuseColor = getDiffuseColor();
	if ( m_pDiffuseColor == NULL ) {
		m_pDiffuseColor = getEmissiveColor();
		if ( m_pDiffuseColor == NULL ) {
			return;
		}
	}

    float b;

    if ( SUCCEEDED( m_pDiffuseColor->get_Blue( &b ))) {
		b = (float)ival/(float)65535;
		m_pDiffuseColor -> put_Blue ( b );
	}
}
void InfoViewManager::setRotation( int ival ) {
	//log_message("in setRotation" );
    ISFRotationObject* m_pDiffuseColor = getRotation(); //rotationList[currentIdx];
	if ( m_pDiffuseColor == NULL ) {
		log_message( "no field found" );
		return;
	}

    float angle;

    if ( SUCCEEDED( m_pDiffuseColor->get_Angle( &angle ))) {
		//log_message( "Should have set it" );
		angle = ((float)ival/(float)65535)*(float)6.28;
		m_pDiffuseColor -> put_Angle( angle );
	}
}
void InfoViewManager::setRotation( int idx, float x, float y, float z, float angle ) {
	//log_message("in setRotation" );
    ISFRotationObject* r = visualObjects[idx]->getRotation();
	if ( r != NULL ) {
		r->put_X( x );
		r->put_Y( y );
		r->put_Z( z );
		r->put_Angle( angle );
	}
}

void InfoViewManager::setIn1( int ival ) {
	log_messagei1( "InfoViewManager::setIn1", ival );
	ISFInt32Object* in1 = visualObjects[ currentIdx ]->getIn1();
	if ( in1 != NULL ) {
		log_message( "and actually calling method!" );
		in1->put_Value( ival );
	}
}

void InfoViewManager::setIns1( char* s ) {
//	log_message1( "InfoViewManager::setIn1", s );
	ISFStringObject* ins1 = visualObjects[ currentIdx ]->getIns1();
	if ( ins1 != NULL ) {
//		log_message( "and actually calling method!" );
		ins1->put_Value( CString(s).AllocSysString() );
	}
}


void InfoViewManager::setIn2( int ival ) {
	//log_messagei1( "InfoViewManager::setIn1", ival );
	ISFInt32Object* in1 = visualObjects[ currentIdx ]->getIn2();
	if ( in1 != NULL ) {
		log_message( "and actually calling method!" );
		in1->put_Value( ival );
	}
}
void InfoViewManager::setIn3( int ival ) {
	//log_messagei1( "InfoViewManager::setIn1", ival );
	ISFInt32Object* in1 = visualObjects[ currentIdx ]->getIn3();
	if ( in1 != NULL ) {
		log_message( "and actually calling method!" );
		in1->put_Value( ival );
	}
}
void InfoViewManager::setIn4( int ival ) {
	//log_messagei1( "InfoViewManager::setIn1", ival );
	ISFInt32Object* in1 = visualObjects[ currentIdx ]->getIn4();
	if ( in1 != NULL ) {
		log_message( "and actually calling method!" );
		in1->put_Value( ival );
	}
}



//
//  Show and hide, this should be re-implemented via a Switch node and
//  this should then send values to a Script node that knows how to
//  switch any of its child nodes on and off.
//

void InfoViewManager::Show() {
	if (( currentIdx >= 0 ) && !visible[currentIdx] ) {
		scaleObject( 10000 );
		visible[currentIdx] = true;
	}
}
void InfoViewManager::Hide() {
	if (( currentIdx >= 0 ) && visible[currentIdx] ) {
		scaleObject( 1 );
		visible[currentIdx] = false;
	}
}


/**
 * ======== InfoViewManager::lookup ========
 *
 *  Purpose:  
 *     what "getIdx" below should have been, straight lookup without side effect
 *     "getIdx" side effect is to set currentIdx as well... need to remove "getIdx"
 *
 *  Input Parameters:
 *     name -- name of the object to look up
 */
int InfoViewManager::lookup( char* name ) {
	//log_message1( "try to find: ", name );
	for ( int i = 0; i < transformIdx; i++ ) {
		//log_message1( "compare to: ", transformNameList[i] );
		if ( strcmp( name, transformNameList[i] ) == 0 ) {
			return( i );
		}
	}
	return( -1 );
}


int InfoViewManager::getIdx( char* name ) {
	//log_message1( "try to find: ", name );
	for ( int i = 0; i < transformIdx; i++ ) {
		//log_message1( "compare to: ", transformNameList[i] );
		if ( strcmp( name, transformNameList[i] ) == 0 ) {
			currentIdx = i;
			return( i );
		}
	}
	return( -1 );
}

void InfoViewManager::MoveUp() {
	//log_message("up");
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];	
		ISFVec3fObject* m_pTranslation = vo->getTranslation();
		if ( m_pTranslation == NULL ) {
			log_message( "UP locationList entry is NULL" );
		} else {
			float y;
			if ( SUCCEEDED ( m_pTranslation -> get_Y ( &y ) ) )	{
			//	log_message("doing it" );
				y += .2f; // move it up on 20 cm
				m_pTranslation -> put_Y ( y );
			} else {
			//	log_message( "not doing it" );
			}
		}
	}
}

void InfoViewManager::MoveDown() {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];	
		ISFVec3fObject* m_pTranslation = vo->getTranslation();
		float y;
		if ( SUCCEEDED ( m_pTranslation -> get_Y ( &y ) ) )	{
			y -= .2f; // move it up on 20 cm
			m_pTranslation -> put_Y ( y );
		}
	}
}

void InfoViewManager::MoveRight() {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];	
		ISFVec3fObject* m_pTranslation = vo->getTranslation();
	    float x;
		if ( m_pTranslation != NULL ) {
		    if ( SUCCEEDED( m_pTranslation->get_X( &x ))) {
				x += .2f; // move it right on 20 cm
				m_pTranslation -> put_X ( x );
			}
		}
	}
}

void InfoViewManager::MoveLeft() {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];	
		ISFVec3fObject* m_pTranslation = vo->getTranslation();
		if ( m_pTranslation == NULL ) {
			log_message("translation is NULL" );
		} else {
			float x;

			if ( SUCCEEDED ( m_pTranslation -> get_X ( &x ) ) ) {
				x -= .2f; // move it right on 20 cm
				m_pTranslation -> put_X ( x );
			}
		}
	}
}

void InfoViewManager::MoveIn() {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];	
		ISFVec3fObject* m_pTranslation = vo->getTranslation();
	    float z;

		if ( SUCCEEDED ( m_pTranslation -> get_Z ( &z ) ) )    {
			z += .2f; // move it forward on 20 cm
			m_pTranslation -> put_Z ( z );
		}
	}
}

void InfoViewManager::MoveOut() {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];	
		ISFVec3fObject* m_pTranslation = vo->getTranslation();
	    float z;

		if ( SUCCEEDED ( m_pTranslation -> get_Z ( &z ) ) )    {
			z -= .2f; // move it forward on 20 cm
			m_pTranslation -> put_Z ( z );
		}
	}
}



char* InfoViewManager::getObjectAt( int offset ) {
	if ( offset < transformIdx ) {
		return( transformNameList[ offset ] );
	} else {
		return( "" );
	}
}

bool InfoViewManager::isVisible( int offset ) {
	if ( offset < transformIdx ) {
		return( visible[ offset ] );
	} else {
		return( false );
	}
}


/**
 * ======== InfoViewManager::SetXmin, SetYmin, SetZmin ========
 *
 *  Purpose:
 *    Set the floating point value for the minimum X/Y/Z location.  Used for
 *    mapping 0-65535 values (from neattools) into actual X/Y/Z locations.
 *
 *  Input Parameters:
 *    valueStr -- string form of minimum float value
 *
 *  Return Value:
 *    none.
 */

void InfoViewManager::SetXmin( char* valueStr ) {
	xMin = (float)atof( valueStr );
	log_messagef( "XMIN now set", xMin );
}
void InfoViewManager::SetYmin( char* valueStr ) {
	yMin = (float)atof( valueStr );
	log_messagef( "YMIN now set", yMin );
}
void InfoViewManager::SetZmin( char* valueStr ) {
	zMin = (float)atof( valueStr );
	log_messagef( "ZMIN now set", zMin );
}

void InfoViewManager::SetResolution( char* valueStr ) {
	int bitResolution = atoi( valueStr );
	resolution = 1;
	for ( int i = 0; i < bitResolution; i++ ) {
		resolution = resolution << 1;
	}
}

/**
 * ======== InfoViewManager::SetXmax ========
 *
 *  Purpose:
 *    Set the floating point value for the maximum X location.  Used for
 *    mapping 0-65535 values (from neattools) into actual X locations.
 *
 *  Input Parameters:
 *    valueStr -- string form of maximum float value
 *
 *  Return Value:
 *    none.
 */

void InfoViewManager::SetXmax( char* valueStr ) {
	xMax = (float)atof( valueStr );
	log_messagef( "XMAX now set", xMax );
}
void InfoViewManager::SetYmax( char* valueStr ) {
	yMax = (float)atof( valueStr );
	log_messagef( "YMAX now set", yMax );
}
void InfoViewManager::SetZmax( char* valueStr ) {
	zMax = (float)atof( valueStr );
	log_messagef( "ZMAX now set", zMax );
}


/**
 * ======== InfoViewManager::rescale ========
 *
 *  Purpose:  
 *    rescale a floating point value, handling decreases in scale
 *    by converting percent to a reversable value.  For example,
 *    increasing size by 5 percent requires a corresponding decrease
 *    in value that goes back from 105% size to 100% size.  This means
 *    a less than 100 percent value coming in gets transferred from
 *    NN/100 to (NN+X=100)/100+X.
 */

float InfoViewManager::rescale( float value, int percent ) {
	float percentAsFloat = (float)(percent)/100;
	if ( percent < 100 ) {
		int x = 100 - percent;
		percentAsFloat = (float)(100)/(float)(100+x);
	}
	return( value * percentAsFloat );
}


void InfoViewManager::setXscale( int neattools_value ) {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];
		ISFVec3fObject* m_pScale = vo->getScale();
		if ( m_pScale != NULL ) {
			float x;

			if ( SUCCEEDED ( m_pScale -> get_X ( &x ) ) ) {
				x = (float)neattools_value/(float)(65535/3);
				m_pScale->put_X( x );
			}
		}
	}
}


/**
 *  Set the Y scale of the current object.
 */
void InfoViewManager::setYscale( int neattools_value ) {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];
		ISFVec3fObject* m_pScale = vo->getScale();
		if ( m_pScale != NULL ) {
			float y;

			if ( SUCCEEDED ( m_pScale -> get_Y ( &y ) ) ) {
				y = (float)neattools_value/(float)(65535/3);
				m_pScale->put_Y( y );
				//log_message("setting object Y scale" );
				vo->scaleNextBaseLocation(y);
			}
		}
	}
}
void InfoViewManager::setZscale( int neattools_value ) {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];
		ISFVec3fObject* m_pScale = vo->getScale();
		if ( m_pScale != NULL ) {
			float z;

			if ( SUCCEEDED ( m_pScale -> get_Z ( &z ) ) ) {
				z = (float)neattools_value/(float)(65535/3);
				m_pScale->put_Z( z );
			}
		}
	}
}
void InfoViewManager::setScale( int neattools_value ) {
	setXscale( neattools_value );
	setYscale( neattools_value );
	setZscale( neattools_value );
}


void InfoViewManager::scaleObjectX( int percent ) {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];
		ISFVec3fObject* m_pScale = vo->getScale();
		if ( m_pScale != NULL ) {
			float x;

			if ( SUCCEEDED ( m_pScale -> get_X ( &x ) ) )	{
				m_pScale -> put_X( rescale( x, percent ));
			}
		}
	}
}

void InfoViewManager::scaleObjectY( int percent ) {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];
		ISFVec3fObject* m_pScale = vo->getScale();
		if ( m_pScale != NULL ) {
			float y;

			if ( SUCCEEDED ( m_pScale -> get_Y ( &y ) ) )	{
				m_pScale -> put_Y( rescale( y, percent ));
			}
		}
	}
}

void InfoViewManager::scaleObjectZ( int percent ) {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];
		ISFVec3fObject* m_pScale = vo->getScale();
		if ( m_pScale != NULL ) {
			float z;

			if ( SUCCEEDED ( m_pScale -> get_Z ( &z ) ) )	{
				m_pScale -> put_Z( rescale( z, percent ));
			}
		}
	}
}

void InfoViewManager::scaleObject( int percent ) {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];
		ISFVec3fObject* m_pScale = vo->getScale();
		if ( m_pScale == NULL ) {
			log_message( "HEY, m_pScale is NULL" );
		} else {
			//log_message( "Seem to have got an m_pScale" );
			float x;
			float y;
			float z;

			if ( SUCCEEDED ( m_pScale -> get_X ( &x ) ) )	{
				m_pScale -> put_X( rescale( x, percent ));
			}
			if ( SUCCEEDED ( m_pScale -> get_Y ( &y ) ) )	{
				y = y * (float)(percent)/100; // move it up on 20 cm
				m_pScale -> put_Y( rescale( y, percent ));
			}
			if ( SUCCEEDED ( m_pScale -> get_Z ( &z ) ) )	{
				m_pScale -> put_Z( rescale( z, percent ));
			}
		}
	}
}

void InfoViewManager::setTransparency( ISFFloatObject* t ) {
	if ( t == NULL ) {
		log_message( "NULL transparency" );
	} else {
		//log_messagei1( "setTransparency, idx", currentIdx );
	}
	visualObjects[ currentIdx ]->setTransparency( t ); 
}


void InfoViewManager::RotateX() {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];
		ISFRotationObject* r = vo->getRotation();
		if ( r != NULL ) {
			r->put_X( 1 );
			r->put_Y( 0 );
			r->put_Z( 0 );
			float angle;

			if ( SUCCEEDED( r->get_Angle( &angle ))) {
				//log_message( "Should have set it" );
				angle += (float).157;
				if ( angle > (float)6.28 ) {
					angle -= (float)6.28;
				}
				r -> put_Angle( angle );
			}
		}
	}
}

void InfoViewManager::RotateY() {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];
		ISFRotationObject* r = vo->getRotation();
		if ( r != NULL ) {
			r->put_X( 0 );
			r->put_Y( 1 );
			r->put_Z( 0 );
			float angle;

			if ( SUCCEEDED( r->get_Angle( &angle ))) {
				//log_message( "Should have set it" );
				angle += (float).157;
				if ( angle > (float)6.28 ) {
					angle -= (float)6.28;
				}
				r -> put_Angle( angle );
			}
		}
	}
}
void InfoViewManager::RotateZ() {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];
		ISFRotationObject* r = vo->getRotation();
		if ( r != NULL ) {
			r->put_X( 0 );
			r->put_Y( 0 );
			r->put_Z( 1 );
			float angle;

			if ( SUCCEEDED( r->get_Angle( &angle ))) {
				//log_message( "Should have set it" );
				angle += (float).157;
				if ( angle > (float)6.28 ) {
					angle -= (float)6.28;
				}
				r -> put_Angle( angle );
			}
		}
	}
}
void InfoViewManager::RotateXcc() {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];
		ISFRotationObject* r = vo->getRotation();
		if ( r != NULL ) {
			r->put_X( 1 );
			r->put_Y( 0 );
			r->put_Z( 0 );
			float angle;

			if ( SUCCEEDED( r->get_Angle( &angle ))) {
				//log_message( "Should have set it" );
				angle -= (float).157;
				if ( angle < (float)0 ) {
					angle += (float)6.28;
				}
				r -> put_Angle( angle );
			}
		}
	}
}
void InfoViewManager::RotateYcc() {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];
		ISFRotationObject* r = vo->getRotation();
		if ( r != NULL ) {
			r->put_X( 0 );
			r->put_Y( 1 );
			r->put_Z( 0 );
			float angle;

			if ( SUCCEEDED( r->get_Angle( &angle ))) {
				//log_message( "Should have set it" );
				angle -= (float).157;
				if ( angle < (float)0 ) {
					angle += (float)6.28;
				}
				r -> put_Angle( angle );
			}
		}
	}
}

void InfoViewManager::SetLabel( char* value ) {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];
		vo->setLabel( value );
	}
}

void InfoViewManager::RotateZcc() {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];
		ISFRotationObject* r = vo->getRotation();
		if ( r != NULL ) {
			r->put_X( 0 );
			r->put_Y( 0 );
			r->put_Z( 1 );
			float angle;

			if ( SUCCEEDED( r->get_Angle( &angle ))) {
				//log_message( "Should have set it" );
				angle -= (float).157;
				if ( angle < (float)0 ) {
					angle += (float)6.28;
				}
				r -> put_Angle( angle );
			}
		}
	}
}


//
//  Track objects added to renderer.
//
//  Assumes global variable "transformName" has name of object being added.
//
void InfoViewManager::addObject( CCubesView* original, INodeObject* o, VisualObject* vo, char* externalName ) {
	//rotationList[ transformIdx ] = pRotation;
	transformNameList[ transformIdx ] = newstrcpy(externalName);
	visible[ transformIdx ] = true;
	value[ transformIdx ] = 0;
	visualObjects[ transformIdx ] = vo;
	vo->setOriginalValues();
	currentIdx = transformIdx;
	transformIdx++;
}

char* InfoViewManager::genRadioBoxes() {
	return( genBoxes("RADIO"));
}

char* InfoViewManager::genCheckBoxes() {
	return( genBoxes("CHECKBOX"));
}


char* InfoViewManager::genBoxes( char* boxType ) {
	char buf[10000];
	buf[0] = 0;
	for ( int i = 0; i < transformIdx; i++ ) {
		char tbuf[200];
		char* s = transformNameList[i];
		sprintf( tbuf, "<input type=\"%s\" name=\"object_%s\">%s<br>\n", boxType, s, s);
//		log_message1( "tbuf checkbox is: ", tbuf );
		strcat( buf, tbuf );
//		log_message1( "buf is: ", buf );
	}
	strcat( buf, "\n\n<script language=\"JavaScript\">\nfunction checkAllViz() {\n" );
	for ( i = 0; i < transformIdx; i++ ) {
		char tbuf[200];
		sprintf( tbuf, "document.vizForm.object_%s.checked = true;\n", transformNameList[i] );
		strcat( buf, tbuf );
	}
	strcat( buf, "}\n\n" );
	strcat( buf, "function uncheckAllViz() {\n" );
	for ( i = 0; i < transformIdx; i++ ) {
		char tbuf[200];
		sprintf( tbuf, "document.vizForm.object_%s.checked = false;\n", transformNameList[i] );
		strcat( buf, tbuf );
	}
	strcat( buf, "}\n\n</script>\n" );

	if ( transformIdx == 0 ) {
		strcpy( buf, "No visual objects are loaded" );
	}
	return( newstrcpy( buf ));
}

