#include "stdafx.h"
#include "VisualObject.h"
#include "log.h"

/**
 *  A VisualObject is a single 3d object in the space.  It has a set of attributes
 *  that are modifiable, along with a linked list of siblings ("next" link) and children.
 */
VisualObject::VisualObject( INodeObject* transform ) {
	script_in1 = NULL;
	script_in2 = NULL;
	script_in3 = NULL;
	script_in4 = NULL;
	script_in5 = NULL;
	pTransformNode = transform;
	embeddedTransform = NULL;
	pScale = NULL;
	pRotation = NULL;
	pLocation = NULL;
	pTransparency = NULL;
	pDiffuseColor = NULL;
	pEmissiveColor = NULL;
	pGroup = NULL;
	pLabel = NULL;
	labelString = NULL;
	pMarker = NULL;
	next = NULL;
	base_set = false;
}


/**
 *  Set the location of an associated object from an incoming scale base.
 *  This is the hardcoded version of what should be a general approach to
 *  relating different objects in the visualization.
 */
void VisualObject::scaleNextBaseLocation( float base ) { 
	if ( next != NULL ) 	
		next->scaleBaseLocation( base );	
}


void VisualObject::scaleBaseLocation( float base ) {
	//log_messagef("scaling base Location", base );
	if ( !base_set ) {
		float x,y,z;
		if ( SUCCEEDED ( pLocation->get_X( &x )) && SUCCEEDED( pLocation->get_Y( &y )) && SUCCEEDED( pLocation->get_Z( &z ))) {
			base_x = x;
			base_y = y;
			base_z = z;
			base_set = true;
			//log_messagef("base_x just set to", base_x );
			//log_messagef( "base_y just set to", base_y );
			//log_messagef( "base_z just set to", base_z );
		}
	}
	if ( base_set ) {
		float x = base_x * base;
		float y = base_y * base;
		float z = base_z * base;
		//log_messagef( "adjusting X to", x );
		//log_messagef( "adjusting Y to", y );
		//log_messagef( "adjusting Z to", z );
		pLocation->put_X( x );
		pLocation->put_Y( y );
		pLocation->put_Z( z );
	}
}

void VisualObject::SetScriptInputs( ISFStringObject* ins1, ISFInt32Object* in1, ISFInt32Object* in2, ISFInt32Object* in3, ISFInt32Object* in4, ISFInt32Object* in5 ) {
	script_ins1 = ins1;
	script_in1 = in1;
	script_in2 = in2;
	script_in3 = in3;
	script_in4 = in4;
	script_in5 = in5;
/*	if ( script_in1 == NULL ) {
		log_message( "No script inputs found" );
	} else {
		log_message( "OK, in VisualObject it it NOT NULL" );
	}*/
}

void VisualObject::SetEmbeddedTransform( INodeObject* xform ) {
	embeddedTransform = xform;
}

void VisualObject::setLabel( char* label ) {
	if ( pLabel != NULL ) {
		CString xx( label );
		pLabel->put_Value( xx.AllocSysString() );
		if ( labelString != NULL ) {
			free( labelString );
		}
		labelString = newstrcpy( label );
	}
}

void VisualObject::setLabelString( char* ls ) { 
	if ( ls != NULL ) {
		if ( labelString != NULL ) 
			free( labelString ); 
		labelString = newstrcpy( ls ); 
	}
}

void VisualObject::setOriginalValues() {
	original_transparency = 0.0f;
	if ( pTransparency != NULL ) {
		pTransparency->get_Value( &original_transparency );
	}
	original_scale_x = 0.0f;
	original_scale_y = 0.0f;
	original_scale_z = 0.0f;
	if ( pScale != NULL ) {
		pScale->get_X( &original_scale_x );
		pScale->get_Y( &original_scale_y );
		pScale->get_Z( &original_scale_z );
	}
	original_rotation_x = 0.0f;
	original_rotation_y = 0.0f;
	original_rotation_z = 0.0f;
	original_rotation_angle = 0.0f;
	if ( pRotation != NULL ) {
		pRotation->get_X( &original_rotation_x );
		pRotation->get_Y( &original_rotation_y );
		pRotation->get_Z( &original_rotation_z );
		pRotation->get_Angle( &original_rotation_angle );
	}
	original_diffuse_red = 0.0f;
	original_diffuse_green = 0.0f;
	original_diffuse_blue = 0.0f;
	if ( pDiffuseColor != NULL ) {
		pDiffuseColor->get_Red( &original_diffuse_red );
		pDiffuseColor->get_Green( &original_diffuse_green );
		pDiffuseColor->get_Blue( &original_diffuse_blue );
	}
	original_emissive_red = 0.0f;
	original_emissive_green = 0.0f;
	original_emissive_blue = 0.0f;
	if ( pEmissiveColor != NULL ) {
		pEmissiveColor->get_Red( &original_emissive_red );
		pEmissiveColor->get_Green( &original_emissive_green );
		pEmissiveColor->get_Blue( &original_emissive_blue );
	}
}

void VisualObject::resetOriginalValues() {
	if ( pTransparency != NULL ) {
		pTransparency->put_Value( original_transparency );
	}
	if ( pScale != NULL ) {
		pScale->put_X( original_scale_x );
		pScale->put_Y( original_scale_y );
		pScale->put_Z( original_scale_z );
	}
	if ( pRotation != NULL ) {
		pRotation->put_X( original_rotation_x );
		pRotation->put_Y( original_rotation_y );
		pRotation->put_Z( original_rotation_z );
		pRotation->put_Angle( original_rotation_angle );
	}
	if ( pDiffuseColor != NULL ) {
		pDiffuseColor->put_Red( original_diffuse_red );
		pDiffuseColor->put_Green( original_diffuse_green );
		pDiffuseColor->put_Blue( original_diffuse_blue );
	}
	if ( pEmissiveColor != NULL ) {
		pEmissiveColor->put_Red( original_emissive_red );
		pEmissiveColor->put_Green( original_emissive_green );
		pEmissiveColor->put_Blue( original_emissive_blue );
	}
}