#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <malloc.h>
#include <string.h>
#include "CommandQueue.h"
#include "log.h"
#include "stdafx.h"


//
// simple utility text processing functions:
//
//   newstrcpy( s ) -- allocate memory for and create a copy of a string
//   skipwhite( s ) -- return pointer to next non-white character
//   skipnonwhite( s ) -- return pointer to next white character
//   zapchar( s, c ) -- look for 'c', replace it with 0, return pointer to next character
//
char* newstrcpy( char* s ) {
  char* result = (char*)malloc( strlen( s ) + 1 );
  strcpy( result, s );
  return( result );
}

char* skipwhite( char* scanner ) {
  while ( *scanner ) {
    if ( isspace( *scanner )) {
      scanner++;
    } else {
      return( scanner );
    }
  }
  return( scanner );
}

char* skipnonwhite( char* scanner ) {
	bool inquote = false;
	while ( *scanner ) {
		if ( *scanner == '"' ) {
			if ( inquote ) {
				inquote = false;
			} else {
				inquote = true;
			}
		}
		if ( isspace( *scanner )) {
			if ( !inquote ) {
				return( scanner );
			}
		}
		scanner++;
	}
	return( scanner );
}

char* zapchar( char* scanner, char z ) {
  while ( *scanner ) {
    if ( *scanner == z ) {
      *scanner = 0;
      return( scanner + 1 );
    } else {
      scanner++;
    }
  }
  return( NULL );
}


//
//  general classes:
//
//     dwTokenizer -- name=value tokens, where "=value" is optional
//     dwLanguageLine -- type name=value ..., where "name=value" is repeated several times
//     dwGrid -- handles grid specific language syntax with default values
//     dwImage -- handles image specific language syntax with default values
//

#define dwMAX 10

class dwTokenizer {
  protected:
    char* token[ dwMAX ];
    char* name[ dwMAX ];
    char* value[ dwMAX ];
    int numberTokens;
  public:
    dwTokenizer( char* line );
    char* getToken( int i ) { return( token[ i ] ); }
    char* getName( int i ) { return( name[ i ] ); }
    char* getValue( int i ) { return( value[ i ] ); }
    int getNumberTokens() { return( numberTokens ); }
};

dwTokenizer::dwTokenizer( char* line ) {
  for ( int i = 0; i < dwMAX; i++ ) {
    token[ i ] = name[ i ] = value[ i ] = NULL;
  }
  numberTokens = 0;
  char* scanner = skipwhite( line );
  while ( *scanner ) {
    token[ numberTokens++ ] = scanner;
    scanner = skipnonwhite( scanner );
    if ( *scanner ) {
      *scanner = 0;
      scanner = skipwhite( scanner + 1 );
    }
  }
  for ( i = 0; i < numberTokens; i++ ) {
    name[ i ] = token[ i ];
    value[ i ] = zapchar( name[ i ], '=' );
  }
}
 

class dwLanguageLine : dwTokenizer {
  private:
    char* dwType;
  public:
    dwLanguageLine( char* line );
    char* getType() { return( dwType ); }
    char* getName( int i ) { return( name[ i ] ); }
    char* getValue( int i ) { return( value[ i ] ); }
    char* getValue( char* name );
};

dwLanguageLine::dwLanguageLine( char* line ) : dwTokenizer( line ) {
  dwType = getToken( 0 );
}

char* dwLanguageLine::getValue( char* name ) {
  int nTokens = getNumberTokens();
  for ( int i = 0; i < nTokens; i++ ) {
    if ( strcmp( name, getName( i )) == 0 ) {
      return( getValue( i ));
    }
  }
  return( NULL );
}

class dwVisibleObject : public dwLanguageLine {
protected:
	// color=r,g,b (where r,g,b is 0-1 range of vlaues)
    char* red;
    char* green;
    char* blue;

	// location=x,y,z
	char* x;
	char* y;
	char* z;

	// name=val
	char* name;
public:
	dwVisibleObject( char* line );
	char* getName() { return( name ); }
};
  
class dwGrid : public dwVisibleObject {
  private:
	// size=AxB where A and B are int values > 0
    int width;
    int height;

  public:
    dwGrid( char* line );
    void generate( CommandQueue* cq );
};

class dwLabel : public dwVisibleObject {
private:

	// text="the test label goes here"
	char* text;

	// scale=n
	char* scale;
public:
	dwLabel( char* line );
	void generate( CommandQueue* cq );
};

class dwView : public dwVisibleObject {
  private:
  public:
    dwView( char* line );
    void generate( CommandQueue* cq );
};

class dwImage: public dwVisibleObject {
  private:
	// image URL
	char* url;

	// size=AxB where A and B are int values > 0
    char* width;
    char* height;

  public:
    dwImage( char* line );
    void generate( CommandQueue* cq );
};

class dwCoordSystem: public dwVisibleObject {
  private:
	// size=XxYxZ > 0
    char* xSize;
    char* ySize;
	char* zSize;

	// resolution=N
	char* resolution;

  public:
    dwCoordSystem( char* line );
    void generate( CommandQueue* cq );
};
class dwObject: public dwVisibleObject {
  private:
	// type=cylinder or sphere
    char* type;
	
	// cylinder only
	char* radius;
	char* height;
	char* orientation;

  public:
    dwObject( char* line );
    void generate( CommandQueue* cq );
};

dwObject::dwObject( char* line ) : dwVisibleObject( line ) {
	type = "cylinder";
	char* otype = getValue( "type" );
	if ( otype != NULL ) {
		type = otype;
	}
	// these are cylinder only
	radius = getValue( "radius" );
	if ( radius == NULL ) {
		radius = "1";
	}
	height = getValue( "height" );
	if ( height == NULL ) {
		height = "1";
	}
	orientation = getValue( "orientation" );
	if ( orientation == NULL ) {
		orientation = "y";
	}
}

void dwObject::generate( CommandQueue* cq) {
  cq->addCommand( Data, "Transform { translation " );
  char lbuf[30];
  sprintf( lbuf, "%s %s %s ", x, y, z );
  cq->addCommand( Data, lbuf );
  if ( strcmp( orientation, "y" ) == 0 ) {
  } else if ( strcmp( orientation, "x" ) == 0 ) {
	sprintf( lbuf, "rotation 0 0 1 1.57" );
	cq->addCommand( Data, lbuf );
  } else if ( strcmp( orientation, "z" ) == 0 ) {
	sprintf( lbuf, "rotation 1 0 0 1.57" );
	cq->addCommand( Data, lbuf );
  }
  cq->addCommand( Data, " children [ Shape { appearance Appearance { " );
  char ibuf[200];
  sprintf( ibuf, " material Material { emissiveColor %s %s %s diffuseColor %s %s %s } } ", red, green, blue, red, green, blue );
  log_message( newstrcpy( ibuf ));
  cq->addCommand( Data, ibuf );
  if ( strcmp( type, "cylinder" ) == 0 ) {
	sprintf( ibuf, " geometry Cylinder { radius %s height %s } ", radius, height );
	cq->addCommand( Data, ibuf );
	log_message( newstrcpy( ibuf ));
  }
  cq->addCommand( Data, " } ] }" );
}

dwCoordSystem::dwCoordSystem( char* line ) : dwVisibleObject( line ) {
	xSize = "1";
	ySize = "1";
	zSize = "1";
	char* size = getValue( "size" );
	if ( size != NULL ) {
		xSize = size;
		ySize = zapchar( xSize, 'x' );
		zSize = zapchar( ySize, 'x' );
	}
	resolution = getValue( "resolution" );
	if ( resolution == NULL ) {
		resolution = "16";
	}
}

void dwCoordSystem::generate( CommandQueue* cq ) {
		char buf[100];
		sprintf( buf, "xmin %s", x );
		cq->addCommand( SetInfo, newstrcpy( buf )); 
		sprintf( buf, "xmax %f", atof( x ) + atof( xSize ));
		cq->addCommand( SetInfo, newstrcpy( buf ));
		sprintf( buf, "ymin %s", y );
		cq->addCommand( SetInfo, newstrcpy( buf ));
		sprintf( buf, "ymax %f", atof( y ) + atof( ySize ));
		cq->addCommand( SetInfo, newstrcpy( buf ));
		sprintf( buf, "zmin %s", z );
		cq->addCommand( SetInfo, newstrcpy( buf ));
		sprintf( buf, "zmax %f", atof( z ) + atof( zSize ));
		cq->addCommand( SetInfo, newstrcpy( buf ));
		sprintf( buf, "resolution %s", resolution );
		cq->addCommand( SetInfo, newstrcpy( buf ));
}

dwImage::dwImage( char* line ) : dwVisibleObject( line ) {
  url = "c:/mindtel/render/textures/aworld.png";
  char* iurl = getValue( "url" );
  if ( iurl != NULL ) {
	  url = iurl;
  }
  width = "1";
  height = "1";
  char* size = getValue( "size" );
  if ( size != NULL ) {
    char* w = size;
    char* h = zapchar( w, 'x' );
	if ( w != NULL ) {
		width = w;
	}
	if ( h != NULL ) {
		height = h;
	}
  }
}

void dwImage::generate( CommandQueue* cq) {
  cq->addCommand( Data, "Transform { translation " );
  char lbuf[30];
  sprintf( lbuf, "%s %s %s ", x, y, z );
  cq->addCommand( Data, lbuf );
  cq->addCommand( Data, " children [ Shape { appearance Appearance { " );
  char ibuf[200];
  sprintf( ibuf, " texture ImageTexture { url \"%s\" } } ", url );
  cq->addCommand( Data, ibuf );
  char coordlocs[100];
  sprintf( coordlocs, "0 0 0, 0 %s 0, %s %s 0, %s 0 0", height, width, height, width );
  cq->addCommand( Data, " geometry IndexedFaceSet { coord Coordinate { point [ " );
  cq->addCommand( Data, coordlocs );
  cq->addCommand( Data, " ] }" );
  cq->addCommand( Data, " coordIndex [ 3 2 1 0 -1 ]  texCoord TextureCoordinate { point [ 0 0 0 1 1 1 1 0 ] } texCoordIndex [ 0 1 2 3 ] }" );
  cq->addCommand( Data, " } ] }" );
}


dwLabel::dwLabel( char* line ) : dwVisibleObject( line ) {
  scale = "1";
  char* tscale = getValue( "scale" );
  if ( tscale != NULL ) {
	  scale = tscale;
  }

  text = "test label";
  char* label = getValue( "text" );
  if ( label != NULL ) {
	  text = label;
  }
}

void dwLabel::generate( CommandQueue* cq) {
  cq->addCommand( Data, "Transform { translation " );
  char lbuf[30];
  sprintf( lbuf, "%s %s %s ", x, y, z );
  cq->addCommand( Data, lbuf );
  cq->addCommand( Data, " scale " );
  sprintf( lbuf, "%s %s %s ", scale, scale, scale );
  cq->addCommand( Data, lbuf );
  cq->addCommand( Data, " children [ Shape { appearance Appearance { " );
  char ibuf[200];
  sprintf( ibuf, " material Material { emissiveColor %s %s %s  diffuseColor %s %s %s } } ", red, green, blue, red, green, blue );
  cq->addCommand( Data, ibuf );
  sprintf( ibuf, " geometry DEF %s_text Text3D { string [ ", name );
  cq->addCommand( Data, ibuf );
  cq->addCommand( Data, text );
  cq->addCommand( Data, " ] }" );
  cq->addCommand( Data, " } " );
  sprintf( ibuf, " DEF Script_%s Script {", name );
  cq->addCommand( Data, ibuf );
  cq->addCommand( Data, " eventIn SFString ins1" );
  cq->addCommand( Data, " eventOut MFString outString" );
  cq->addCommand( Data, " url \"javascript:" );
  cq->addCommand( Data, "   function ins1(value) {" );
  cq->addCommand( Data, "     outString[0] = value;" );
  cq->addCommand( Data, "  }\" }" );
  sprintf( ibuf, " ROUTE Script_%s.outString TO %s_text.set_string", name, name );
  cq->addCommand( Data, ibuf );
  cq->addCommand( Data, " ] }" );
}

dwView::dwView( char* line ) : dwVisibleObject( line ) {
}


void dwView::generate( CommandQueue* cq) {
  cq->addCommand( Data, "Viewpoint { " );
  char buf[100];
  sprintf( buf, "position %s %s %s", x, y, z );
  cq->addCommand( Data, buf );
  cq->addCommand( Data, "} ");
}

dwVisibleObject::dwVisibleObject( char* line ) : dwLanguageLine( line ) {
  x = "0";
  y = "0";
  z = "0";
  char* location = getValue( "location" );
  if ( location != NULL ) {
    char* xs = location;
    char* ys = zapchar( xs, ',' );
	char* zs = zapchar( ys, ',' );
	if ( xs != NULL ) {
		x = xs;
	}
	if ( ys != NULL ) {
		y = ys;
	}
	if ( zs != NULL ) {
		z = zs;
	}
  }

  red = ".5";
  green = ".5";
  blue = ".5";
  char* rgb = getValue( "color" );
  if ( rgb != NULL ) {
    char* r = rgb;
    char* g = zapchar( r, ',' );
    char* b = zapchar( g, ',' );
    if ( r != NULL ) {
      red = r;
    }
    if ( g != NULL ) {
      green = g;
    }
    if ( b != NULL ) {
      blue = b;
    }
  }
  name = "no_name";
  char* lname = getValue( "name" );
  if ( lname != NULL ) {
	  name = lname;
  }
}

dwGrid::dwGrid( char* line ) : dwVisibleObject( line ) {

  width = 10;
  height = 10;
  char* size = getValue( "size" );
  if ( size != NULL ) {
    char* w = size;
    char* h = zapchar( w, 'x' );
    int wint = atoi( w );
    int hint = atoi( h );
    if ( wint > 0 ) {
      width = wint;
    }
    if ( hint > 0 ) {
      height = hint;
    }
  }
}

void dwGrid::generate( CommandQueue* cq) {
  int i, j;
  cq->addCommand( Data, "Transform { " );
  char cbuf[30];
  sprintf( cbuf, "translation %s %s %s ", x, y, z );
  cq->addCommand( Data, cbuf );
  cq->addCommand( Data, " children [ Shape { appearance Appearance { material Material { " );
  sprintf( cbuf, " emissiveColor %s %s %s } }", red, green, blue );
  cq->addCommand( Data, cbuf );
  cq->addCommand( Data, " geometry IndexedLineSet { coord Coordinate { point [\n" );
  for ( i = 0; i < width + 1; i++ ) {
    for ( j = 0; j < height + 1; j++ ) {
      char buf[50];
      sprintf( buf, "%d %d 0, ", i, j );
      cq->addCommand( Data, buf );
    }
  }
  cq->addCommand( Data, " ] } coordIndex [\n" );
  for ( j = 0; j < width + 1; j++ ) {
    for ( i = 0; i < height; i++ ) {
      char buf[50];
      sprintf( buf, "%d %d -1, ", (height+1)*j + i, (height+1)*j + i + 1 );
      cq->addCommand( Data, buf );
    }
  }
  for ( i = 0; i < width; i++ ) {
    for ( j = 0; j < height + 1; j++ ) {
      char buf[50];
      sprintf( buf, "%d %d -1, ", i*(height+1) + j, (i+1)*(height+1) + j );
	  cq->addCommand( Data, buf );
    }
  }
  cq->addCommand( Data, " ] } } ] }" );
}

void dwHandle( CommandQueue* cq, char* line ) {
	if ( strncmp( line, "grid", 4 ) == 0 ) {
		dwGrid dwg( line );
		cq->suspend();
		cq->addCommand( StartData, dwg.getName() );
		dwg.generate( cq );
		cq->addCommand( EndData, NULL );
		cq->resume();
	} else if ( strncmp( line, "view", 4 ) == 0 ) {
		dwView dwv( line );
		cq->suspend();
		cq->addCommand( StartData, dwv.getName() );
		dwv.generate( cq );
		cq->addCommand( EndData, NULL );
		cq->resume();
	} else if ( strncmp( line, "image", 5 ) == 0 ) {
		dwImage dwi( line );
		cq->suspend();
		cq->addCommand( StartData, dwi.getName() );
		dwi.generate( cq );
		cq->addCommand( EndData, NULL );
		cq->resume();
	} else if ( strncmp( line, "label", 5 ) == 0 ) {
		dwLabel dwl( line );
		cq->suspend();
		cq->addCommand( StartData, dwl.getName() );
		dwl.generate( cq );
		cq->addCommand( EndData, NULL );
		cq->resume();
	} else if ( strncmp( line, "link", 4 ) == 0 ) {
		cq->suspend();
		cq->addCommand( Link, line + 5 );
		cq->resume();
	} else if ( strncmp( line, "coordsystem", 11 ) == 0 ) {
		dwCoordSystem dwcs( line + 12 );
		cq->suspend();
		dwcs.generate( cq );
		cq->resume();
	} else if ( strncmp( line, "object", 6 ) == 0 ) {
		dwObject dwo( line + 7 );
		cq->suspend();
		cq->addCommand( StartData, dwo.getName() );
		dwo.generate( cq );
		cq->addCommand( EndData, NULL );
		cq->resume();
	}
}